using System.Collections.Generic;
using System.Text.RegularExpressions;
using AndroidWpf.Core.Messages;
using Cirrious.CrossCore;
using Cirrious.MvvmCross.Plugins.Email;
using Cirrious.MvvmCross.Plugins.Messenger;
using Cirrious.MvvmCross.ViewModels;

namespace AndroidWpf.Core.ViewModels
{
    public class FirstViewModel
        : MvxViewModel
    {

        #region fields

        private readonly IMvxComposeEmailTask emailTask;
        private readonly IMvxMessenger messenger;
        private List<string> errorList = new List<string> { "Please compose your mail" };
        private bool areDataOk;
        private string mailToError;
        private string mailSubjectError;
        private string mailBodyError;
        private string mailTo;
        private string mailCc;
        private string mailSubject;
        private string mailBody;
        private bool isHtml;

        #endregion

        #region consts

        const string ValidEmailPattern = @"^(?!\.)(""([^""\r\\]|\\[""\r\\])*""|"
                                 + @"([-a-z0-9!#$%&'*+/=?^_`{|}~]|(?<!\.)\.)*)(?<!\.)"
                                 + @"@[a-z0-9][\w\.-]*[a-z0-9]\.[a-z][a-z\.]*[a-z]$";
        const string Cbe = " can't be empty.";

        #endregion

        #region properties

        public string MailTo
        {
            get { return mailTo; }
            set
            {
                if (value == mailTo) return;
                mailTo = value;
                RaisePropertyChanged(() => MailTo);
                checkMailData();
            }
        }

        public string MailCc
        {
            get { return mailCc; }
            set
            {
                if (value == mailCc) return;
                mailCc = value;
                RaisePropertyChanged(() => MailCc);
                checkMailData();
            }
        }

        public string MailSubject
        {
            get { return mailSubject; }
            set
            {
                if (value == mailSubject) return;
                mailSubject = value;
                RaisePropertyChanged(() => MailSubject);
                checkMailData();
            }
        }

        public string MailBody
        {
            get { return mailBody; }
            set
            {
                if (value == mailBody) return;
                mailBody = value;
                RaisePropertyChanged(() => MailBody);
                checkMailData();
            }
        }

        public bool IsHtml
        {
            get { return isHtml; }
            set
            {
                if (value == isHtml) return;
                isHtml = value;
                checkMailData();
            }
        }

        #endregion

        #region error management properties
 
        public List<string> ErrorList
        {
            get { return errorList; }
            set
            {
                errorList = value;
                RaisePropertyChanged(() => ErrorList);
            }
        }

        public bool AreDataOk
        {
            get { return areDataOk; }
            set
            {
                areDataOk = value;
                RaisePropertyChanged(() => AreDataOk);
            }
        }

        public string MailToError
        {
            get { return mailToError; }
            set
            {
                if (value == mailToError) return;
                mailToError = value;
                RaisePropertyChanged(() => MailToError);
            }
        }

        public string MailSubjectError
        {
            get { return mailSubjectError; }
            set
            {
                if (value == mailSubjectError) return;
                mailSubjectError = value;
                RaisePropertyChanged(() => MailSubjectError);
            }
        }

        public string MailBodyError
        {
            get { return mailBodyError; }
            set
            {
                if (value == mailBodyError) return;
                mailBodyError = value;
                RaisePropertyChanged(() => MailBodyError);
            }
        }

        #endregion

        #region commands

        public IMvxCommand SendMailCommand { get; private set; }

        #endregion

        #region Ctor

        public FirstViewModel(IMvxComposeEmailTask emailTask, IMvxMessenger messenger)
        {
            this.emailTask = emailTask;
            this.messenger = messenger;
            SendMailCommand = new MvxCommand(sendMail);
        }

        #endregion

        #region private methods

        private void sendMail()
        {
            checkMailData();
            if (AreDataOk) emailTask.ComposeEmail(MailTo, mailCc, MailSubject, MailBody, IsHtml);
            Mvx.Trace("SENT. TO:{0} CC:{1} SUBJECT:{2} HTML:{3} BODY:{4}", MailTo, MailCc, MailSubject, IsHtml, MailBody);
        }

        private void checkMailData()
        {
            var list = new List<string>();
            mailToError = null;


            if (string.IsNullOrWhiteSpace(mailTo))
            {
                const string s = "Recipient" + Cbe;
                list.Add(s);
                MailToError = s;
            }
            if (string.IsNullOrWhiteSpace(mailSubject))
            {
                const string s = "Subject" + Cbe;
                list.Add(s);
                MailSubjectError = s;
            }
            if (string.IsNullOrWhiteSpace(mailBody))
            {
                const string s = "Body" + Cbe;
                list.Add(s);
                MailBodyError = s;
            }

            if (!string.IsNullOrWhiteSpace(mailTo))
            {
                var v = Regex.IsMatch(MailTo, ValidEmailPattern);
                if (!v)
                {
                    const string s = "Recipient address is not valid";
                    list.Add(s);
                    MailToError = s;
                }
            }

            ErrorList = list;
            AreDataOk = ErrorList.Count == 0;

            messenger.Publish(new ErrorMessage(this, ErrorList));
        }

        #endregion


    }
}
